package demo;

import java.io.IOException;
import java.util.Collections;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.PartialResponseWriter;
import javax.faces.context.PartialViewContext;
import javax.faces.context.PartialViewContextWrapper;
import javax.faces.event.PhaseId;

public class PartialViewContextImpl extends PartialViewContextWrapper {

    private PartialViewContext partialViewContext;

    public PartialViewContextImpl(PartialViewContext partialViewContext) {
        super();
        this.partialViewContext = partialViewContext;
    }

    public PartialViewContext getWrapped() {
        return partialViewContext;
    }

    @Override
    public void setPartialRequest(boolean isPartialRequest) {
        getWrapped().setPartialRequest(isPartialRequest);
    }
    
    @Override
    public void processPartial(PhaseId phaseId) {
        if (phaseId != PhaseId.RENDER_RESPONSE) {
            super.processPartial(phaseId);
        } else {
            processPartialRenderPhase();
        }
    }
    
    private void renderState(FacesContext context) throws IOException {
        if (!context.getViewRoot().isTransient()) {
            // Get the view state and write it to the response..
            PartialViewContext pvc = context.getPartialViewContext();
            PartialResponseWriter writer = pvc.getPartialResponseWriter();
            writer.startUpdate(PartialResponseWriter.VIEW_STATE_MARKER);
            String state = context.getApplication().getStateManager().getViewState(context);
            writer.write(state);
            writer.endUpdate();
        }
    }

    protected void processPartialRenderPhase() {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        PartialViewContext pvc = facesContext.getPartialViewContext();

        try {
            PartialResponseWriter writer = pvc.getPartialResponseWriter();
            facesContext.setResponseWriter(writer);

            ExternalContext exContext = facesContext.getExternalContext();
            exContext.setResponseContentType("text/xml");
            exContext.addResponseHeader("Cache-Control", "no-cache");
            writer.startDocument();

            renderState(facesContext);

            writer.startExtension(Collections.<String, String>emptyMap());
            
            writer.startElement("data", null);
            writer.endElement("data");
            
            writer.endExtension();
            
            writer.endDocument();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
}
